/**
*	@file SensorCommunicationLib.cpp
*	@brief contains the SensorCommunicationLib class mathods implementation. This class can be used to ease
*	the programming of an Arduino sketch read data from sensors and transmit them as a stream of csv data. 
*
*	@author	Antonio Cafiero
*
*	@date 11/10/2019
*/


/*
 Copyright 2019, Toni Cafiero, IoThingsWare 
 MIT License terms detailed in LICENSE.txt 
*/
#include <Arduino.h>
#include <SensorCommunicationLib.h>


//Serial data variables
const char kDelimiter = ',';
const int kSerialInterval = 1000;
unsigned long serialPreviousTime;
Smoothed <float> avgSensors[MAXNUMOFSENSORS];



/**
*	This method is the constructor of the SensorCommunicationLib
*	@author	Antonio Cafiero
*	@param NumberOfSensors The number of sensors value to transmit as a stream of csv (max 10)
*	@date 11/10/2019
*/ 
SensorCommunicationLib::SensorCommunicationLib(int NumberOfSensors)
{
	kNumberOfSensors=NumberOfSensors;
	for(int i=0; i < NumberOfSensors; i++) {
		avgSensors[i].begin(SMOOTHED_AVERAGE, 10); 
		avgSensors[i].clear();
	}
} 
 
void SensorCommunicationLib::Send()
{
  int i;
  for(i = 0; i < kNumberOfSensors-1; i++){
      Serial.printf("%.2f", avgSensors[i].get());
      Serial.print(kDelimiter);
  }
  Serial.printf("%.2f", avgSensors[i].get());
  Serial.println();
}

void SensorCommunicationLib::Tentative()
{
  if((millis() - serialPreviousTime) > kSerialInterval) 
  {
    serialPreviousTime = millis();
    Send(); 
  }
}

/**
*	This method must be called into the loop()
*	@author	Antonio Cafiero
*	@param getSensorData The pointer of the callback routine fo sensors data reading.
*	@date 11/10/2019
*/ 
void SensorCommunicationLib::processSensors(void (*getSensorData)()) //call in loop()
{
	(*getSensorData)();
	for(int i = 0; i < kNumberOfSensors; i++){
	avgSensors[i].add(Sensors[i]);
	}
	delay(10);
	Tentative();
}
